/*
 * Main.fx
 */

package pausetdemo;

import javafx.animation.transition.PauseTransition;

import javafx.scene.CustomNode;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.Scene;

import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;

import javafx.scene.paint.Color;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Paint;
import javafx.scene.paint.Stop;

import javafx.scene.shape.Rectangle;

import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.scene.text.TextOrigin;

import javafx.stage.Stage;

def BACKGROUND_PAINT = LinearGradient
{
    startX: 0.0
    startY: 0.0
    endX: 0.0
    endY: 1.0
    stops:
    [
        Stop { offset: 0.0 color: Color.BLACK },
        Stop { offset: 1.0 color: Color.BLUEVIOLET }
    ]
}

Stage
{
    title: "PauseTransition Demo"
    width: 300
    height: 200

    var scene: Scene
    scene: scene = Scene
    {
        fill: BACKGROUND_PAINT

        var button: Button;
        content: button = Button
        {
            x: bind (scene.width-100)/2
            y: bind (scene.height-40)/2
            width: 100
            height: 40
            arcWidth: 10
            arcHeight: 10

            text: "OK"
            font: Font
            {
                name: "Arial BOLD"
                size: 16
            }

            fill: Color.DARKBLUE
            fillRollover: Color.MEDIUMBLUE
            fillPressed: Color.LIGHTBLUE
            borderColor: Color.WHITE
            textColor: Color.WHITE

            action: function (): Void
            {
                button.text = if (button.text == "OK")
                                  "OKAY"
                              else
                                  "OK";

                button.fillRollover = if (button.fill == Color.DARKBLUE)
                                          Color.MEDIUMVIOLETRED
                                      else
                                          Color.MEDIUMBLUE;

                button.fillPressed = if (button.fill == Color.DARKBLUE)
                                         Color.PALEVIOLETRED
                                     else
                                         Color.LIGHTBLUE;

                button.fill = if (button.fill == Color.DARKBLUE)
                                  Color.BLUEVIOLET
                              else
                                  Color.DARKBLUE
            }
        }
    }
}

class Button extends CustomNode
{
    public var text: String;
    public var font: Font;

    public var x: Number;
    public var y: Number;
    public var width: Number;
    public var height: Number;
    public var arcWidth: Number;
    public var arcHeight: Number;

    public var fill: Paint on replace oldFill
    {
        if (curFill == oldFill) curFill = fill
    }

    public var fillPressed: Paint on replace oldFillPressed
    {
        if (curFill == oldFillPressed) curFill = fillPressed
    }

    public var fillRollover: Paint on replace oldFillRollover
    {
        if (curFill == oldFillRollover) curFill = fillRollover
    }

    public var borderColor: Paint;
    public var textColor: Paint;

    public var action: function (): Void;

    var curFill: Paint;

    public override function create (): Node
    {
        Group
        {
            var r: Rectangle on replace { if (r != null) r.requestFocus () }
            var t: Text
            content:
            [
                r = Rectangle
                {
                    x: bind x
                    y: bind y
                    width: bind width
                    height: bind height
                    arcWidth: bind arcWidth
                    arcHeight: bind arcHeight
                    fill: bind curFill
                    stroke: bind borderColor

                    onMouseEntered: function (me: MouseEvent): Void
                    {
                        curFill = fillRollover
                    }

                    onMouseExited: function (me: MouseEvent): Void
                    {
                        curFill = fill
                    }

                    onMousePressed: function (me: MouseEvent): Void
                    {
                        curFill = fillPressed
                    }

                    onMouseReleased: function (me: MouseEvent): Void
                    {
                        curFill = if (r.contains (me.x, me.y))
                                      fillRollover
                                  else
                                      fill
                    }

                    onMouseClicked: function (e: MouseEvent): Void
                    {
                        action ()
                    }

                    onKeyPressed: function (e: KeyEvent): Void
                    {
                        if (e.code != KeyCode.VK_SPACE)
                            return;

                        def pause = PauseTransition
                        {
                            duration: 68ms
                            action: function (): Void
                            {
                                curFill = fill;
                                action ()
                            }
                        }
                        curFill = fillPressed;
                        pause.play ()
                    }
                }
                t = Text
                {
                    content: bind text
                    font: bind font
                    textOrigin: TextOrigin.TOP
                    translateX: bind x+(r.layoutBounds.width-
                                        t.layoutBounds.width)/2
                    translateY: bind y+(r.layoutBounds.height-
                                        t.layoutBounds.height)/2
                    fill: bind textColor
                }
            ]
        }
    }
}