/*
 * Main.fx
 */

package scaletdemo;

import javafx.animation.transition.ScaleTransition;

import javafx.scene.CustomNode;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.Scene;

import javafx.scene.image.Image;
import javafx.scene.image.ImageView;

import javafx.scene.input.MouseEvent;

import javafx.scene.paint.Color;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;

import javafx.scene.shape.Rectangle;

import javafx.stage.Stage;

def BACKGROUND_PAINT = LinearGradient
{
    startX: 0.0
    startY: 0.0
    endX: 1.0
    endY: 1.0
    stops:
    [
        Stop { offset: 0.0 color: Color.YELLOW },
        Stop { offset: 0.5 color: Color.ORANGE },
        Stop { offset: 1.0 color: Color.PINK }
    ]
}

def NUMIMAGES = 5;

Stage
{
    title: "ScaleTransition Demo"
    width: 750
    height: 400

    var scene: Scene
    scene: scene = Scene
    {
        fill: BACKGROUND_PAINT
        
        var t: ThumbNail
        content: Group
        {
            content: for (i in [1..NUMIMAGES])
                          t = ThumbNail
                          {
                              translateX: bind (scene.width+10-
                                                NUMIMAGES*t.layoutBounds.width-
                                                10*(NUMIMAGES-1))/2+
                                                (i-1)*(t.layoutBounds.width+10)
                              translateY: bind (scene.height-
                                                t.layoutBounds.height)/2
                              width: 100
                              height: 100
                              imageName: "photo{i}"
                          }
        }
    }
}

class ThumbNail extends CustomNode
{
    public var width: Number;
    public var height: Number;
    public var imageName: String;
    
    var view: ImageView;

    def stBig = ScaleTransition
                {
                    node: this
                    fromX: 1.0
                    fromY: 1.0
                    toX: 2.0
                    toY: 2.0
                    duration: 1s
                }

    def stSmall = ScaleTransition
                  {
                      node: this
                      fromX: 2.0
                      fromY: 2.0
                      toX: 1.0
                      toY: 1.0
                      duration: 1s
                  }

    public override function create (): Node
    {
        Group
        {
            content:
            [
                Rectangle
                {
                    x: 0
                    y: 0
                    width: bind width
                    height: bind height
                    arcWidth: 5
                    arcHeight: 5
                    strokeWidth: 4
                    stroke: Color.GRAY
                }
                view = ImageView
                {
                    image: Image { url: "{__DIR__}res/{imageName}.jpg"}
                    x: 4
                    y: 4
                    fitWidth: bind width-8
                    fitHeight: bind height-8
                }
            ]

            blocksMouse: true

            onMouseEntered: function (me: MouseEvent): Void
            {
                if (stSmall.running)
                {
                    stSmall.stop ();
                    stBig.fromX = stSmall.node.scaleX;
                    stBig.fromY = stSmall.node.scaleY
                }
                else
                {
                    stBig.fromX = 1.0;
                    stBig.fromY = 1.0
                }

                stBig.toX = 2.0;
                stBig.toY = 2.0;

                stBig.node.toFront ();
                stBig.playFromStart ()
            }

            onMouseExited: function (me: MouseEvent): Void
            {
                if (stBig.running)
                {
                    stBig.stop ();
                    stSmall.fromX = stBig.node.scaleX;
                    stSmall.fromY = stBig.node.scaleY
                }
                else
                {
                    stSmall.fromX = 2.0;
                    stSmall.fromY = 2.0
                }
                stSmall.toX = 1.0;
                stSmall.toY = 1.0;

                stSmall.playFromStart ()
            }
        }
    }
}